import os


class BaseConfig:
    SOURCE_WAIT_INTERVAL = int(os.environ.get("SOURCE_WAIT_INTERVAL", 5000))
    AUTO_LOAD_IMAGES = bool(os.environ.get("AUTO_LOAD_IMAGES", True))
    JAVASCRIPT_ENABLED = bool(os.environ.get("JAVASCRIPT_ENABLED", True))
    JAVASCRIPT_CAN_OPEN_WINDOWS = bool(os.environ.get("JAVASCRIPT_CAN_OPEN_WINDOWS", False))
    LOCAL_STORAGE_ENABLED = bool(os.environ.get("LOCAL_STORAGE_ENABLED", True))
    LOCAL_CONTENT_CAN_ACCESS_REMOTE_URLS = bool(os.environ.get("LOCAL_CONTENT_CAN_ACCESS_REMOTE_URLS", True))
    LOCAL_CONTENT_CAN_ACCESS_FILE_URLS = bool(os.environ.get("LOCAL_CONTENT_CAN_ACCESS_FILE_URLS", True))
    ERROR_PAGES_ENABLED = bool(os.environ.get("ERROR_PAGES_ENABLED", False))
    PLUGINS_ENABLED = bool(os.environ.get("PLUGINS_ENABLED", False))
    WEBGL_ENABLED = bool(os.environ.get("WEBGL_ENABLED", True))
    ALLOW_RUNNING_INSECURE_CONTENT = bool(os.environ.get("ALLOW_RUNNING_INSECURE_CONTENT", False))
    ALLOW_GEOLOCATION_ON_INSECURE_ORIGINS = bool(os.environ.get("ALLOW_GEOLOCATION_ON_INSECURE_ORIGINS", False))
    SHOW_SCROLL_BARS = bool(os.environ.get("SHOW_SCROLL_BARS", False))
    DNS_PREFETCH_ENABLED = bool(os.environ.get("DNS_PREFETCH_ENABLED", False))
    USER_AGENT_LIST = [
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.246"
        "Mozilla/5.0 (X11; CrOS x86_64 8172.45.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.64 Safari/537.36",
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9",
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36",
        "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1",
        "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36",
    ]



from PySide2.QtCore import QUrl, QTimer, Qt
from PySide2.QtWebEngineWidgets import QWebEngineView, QWebEngineProfile, QWebEnginePage
from PySide2.QtWebEngineWidgets import QWebEngineSettings as ws
from PySide2.QtWebEngineCore import QWebEngineHttpRequest
from PySide2.QtWidgets import QDialog
from PySide2.QtNetwork import QNetworkCookie
from Py2Web.config import BaseConfig as bconf
import random


class Py2WebBrowser(QDialog):
    def __init__(self):
        super(Py2WebBrowser, self).__init__()
        self.pwb = QWebEngineView()
        self.pwb.setAttribute(Qt.WA_DeleteOnClose, True)

        self.raw_cookies = []
        self.cookie_list = []

        self.req_obj = QWebEngineHttpRequest()
        self.source_timer = QTimer()

        profile = QWebEngineProfile("pyweb", self.pwb)
        profile.setHttpUserAgent(random.choice(bconf.USER_AGENT_LIST))
        cookie_store = profile.cookieStore()

        cookie_store.cookieAdded.connect(self._on_cookie)

        wp = QWebEnginePage(profile, self.pwb)
        self.pwb.setPage(wp)

        self._settings()
        self.pwb.show()

    def _settings(self):
        s = self.pwb.settings()
        s.setAttribute(ws.AutoLoadImages, bconf.AUTO_LOAD_IMAGES)
        s.setAttribute(ws.JavascriptEnabled, bconf.JAVASCRIPT_ENABLED)
        s.setAttribute(ws.JavascriptCanOpenWindows, bconf.JAVASCRIPT_CAN_OPEN_WINDOWS)
        s.setAttribute(ws.LocalStorageEnabled, bconf.LOCAL_STORAGE_ENABLED)
        s.setAttribute(ws.LocalContentCanAccessRemoteUrls, bconf.LOCAL_CONTENT_CAN_ACCESS_REMOTE_URLS)
        s.setAttribute(ws.LocalContentCanAccessFileUrls, bconf.LOCAL_CONTENT_CAN_ACCESS_FILE_URLS)
        s.setAttribute(ws.ErrorPageEnabled, bconf.ERROR_PAGES_ENABLED)
        s.setAttribute(ws.PluginsEnabled, bconf.PLUGINS_ENABLED)
        s.setAttribute(ws.WebGLEnabled, bconf.WEBGL_ENABLED)
        s.setAttribute(ws.AllowRunningInsecureContent, bconf.ALLOW_RUNNING_INSECURE_CONTENT)
        s.setAttribute(ws.AllowGeolocationOnInsecureOrigins, bconf.ALLOW_GEOLOCATION_ON_INSECURE_ORIGINS)
        s.setAttribute(ws.ShowScrollBars, bconf.SHOW_SCROLL_BARS)
        s.setAttribute(ws.DnsPrefetchEnabled, bconf.DNS_PREFETCH_ENABLED)

    def _get_page_source(self):
        if self.pwb.loadFinished:
            self.pwb.page().toHtml(self._page_to_var)
            self.pwb.page().runJavaScript(self.s)

    def _page_to_var(self, html):
        self.page_source = html
        self.source_timer.stop()
        self._to_json()
        self._return()

    def _on_cookie(self, cookie):
        for i in self.raw_cookies:
            if i.hasSameIdentifier(cookie):
                return
        self.raw_cookies.append(QNetworkCookie(cookie))

    def _to_json(self):
        for i in self.raw_cookies:
            data = {
                "name": bytearray(i.name()).decode(),
                "domain": i.domain(),
                "value": bytearray(i.value()).decode(),
                "path": i.path(),
                "expireData": i.expirationDate().toString(),
                "secure": i.isSecure(),
                "httpOnly": i.isHttpOnly()
            }
            self.cookie_list.append(data)

    def _return(self):
        self.return_dict = {
            "url": str(self.req_obj.url().toString()),
            "cookies": self.cookie_list,
            "content": str(self.page_source)
        }
        self.accept()

    def get(self, url: str, script: str = None):
        self.s = script
        self.req_obj.setUrl(QUrl().fromUserInput(url))

        self.source_timer.setInterval(bconf.SOURCE_WAIT_INTERVAL)
        self.source_timer.timeout.connect(self._get_page_source)
        self.source_timer.start()

        self.pwb.page().profile().cookieStore().deleteAllCookies()

        self.pwb.load(self.req_obj)



import sys


class VirtualDisplay:
    def __init__(self):
        try:
            from xvfbwrapper import Xvfb
            self.V_DISPLAY = Xvfb()
        except ImportError:
            self.V_DISPLAY = None

    def init_xvfb(self):
        if not sys.platform.startswith("linux"):
            return
        self.V_DISPLAY.start()

    def stop_xvfb(self):
        if not sys.platform.startswith("linux"):
            return
        self.V_DISPLAY.stop()



from setuptools import setup, find_packages


setup(
    name='Py2Web',
    version='0.0.1.dev1',
    description='A python package to perform HTTP requests to websites and render their content.',
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url='https://github.com/iiestIT/Py2Web',
    author='iiestIT',
    author_email='it.iiest.de@gmail.com',
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Topic :: Internet :: WWW/HTTP :: Browsers",
        "License :: OSI Approved :: MIT License"
    ],
    keywords='pyside2 browser engine rendering',
    python_requires='>=3.8',
    packages=find_packages(),
    install_requires=open("requirements.txt").read().splitlines()
)



from PySide2.QtCore import Qt
from PySide2.QtWidgets import QDialog
from Py2Web.browser import Py2WebBrowser
from Py2Web.utils.xvfb import VirtualDisplay


def get(url: str):
    # vd = VirtualDisplay()
    # vd.init_xvfb()
    pw = Py2WebBrowser()
    pw.get(url)
    pw.setAttribute(Qt.WA_DeleteOnClose, True)
    pw.setAttribute(Qt.WA_DontShowOnScreen, True)
    if pw.exec_() == QDialog.Accepted:
        # vd.stop_xvfb()
        return pw.return_dict
